import httpx
import json

class LoggingAsyncHttpClient(httpx.AsyncClient):
    async def send(self, request, *args, **kwargs):
        # --- REQUEST ---
        print("=== INTERCEPTED ASYNC HTTP REQUEST ===")
        print(request.method, request.url)
        # Be careful: this includes auth headers if you print everything.
        print("Headers:", dict(request.headers))

        # request.content is bytes or None for JSON requests
        if request.content:
            try:
                body = request.content.decode("utf-8")
            except Exception:
                body = str(request.content)
            print("Body:", body)

        # Perform actual request
        response = await super().send(request, *args, **kwargs)

        # --- RESPONSE ---
        print("=== INTERCEPTED ASYNC HTTP RESPONSE ===")
        print("Status:", response.status_code)
        
        try:
            print("Body:", response.text)
        except Exception:
            print("Body: <unprintable>")

        return response


from abc import ABC, abstractmethod
from typing import List
from foundationallm.config import Configuration, UserIdentity
from foundationallm.langchain.common import FoundationaLLMWorkflowBase
from foundationallm.models.agents import (
    AgentTool,
    GenericAgentWorkflow,
    ExternalAgentWorkflow
)
from foundationallm.operations import OperationsManager
from foundationallm.plugins import PluginManagerTypes

class WorkflowPluginManagerBase(ABC):
    """
    The base class for all workflow plugin managers.
    """
    def __init__(self):
        
        self.plugin_manager_type = PluginManagerTypes.WORKFLOWS

    @abstractmethod
    def create_workflow(self,
        workflow_config: GenericAgentWorkflow | ExternalAgentWorkflow,
        objects: dict,
        tools: List[AgentTool],
        operations_manager: OperationsManager,
        user_identity: UserIdentity,
        config: Configuration,
        intercept_http_calls: bool = False) -> FoundationaLLMWorkflowBase:
        """
        Create a workflow instance based on the given configuration and tools.
        Parameters
            ----------
            workflow_config : GenericAgentWorkflow | ExternalAgentWorkflow
                The workflow assigned to the agent.
            objects : dict
                The exploded objects assigned from the agent.
            tools : List[AgentTool]
                The tools assigned to the agent.
            user_identity : UserIdentity
                The user identity of the user initiating the request.
            config : Configuration
                The application configuration for FoundationaLLM.
        """
        pass

    @abstractmethod
    def refresh_tools(self):
        pass



from typing import Any, Self, Literal
from foundationallm.langchain.exceptions import LangChainException
from foundationallm.utils import object_utils
from .agent_workflow_base import AgentWorkflowBase

class LangChainExpressionLanguageAgentWorkflow(AgentWorkflowBase):
    """
    The configuration for a LangChain Expression Language agent workflow.
    """
    type: Literal["langchain-expression-language-workflow"] = "langchain-expression-language-workflow"
    
   
    @staticmethod
    def from_object(obj: Any) -> Self:

        workflow: LangChainExpressionLanguageAgentWorkflow = None

        try:
            workflow = LangChainExpressionLanguageAgentWorkflow(**object_utils.translate_keys(obj))
        except Exception as e:
            raise LangChainException(f"The LangChain Expression Language Agent Workflow object provided is invalid. {str(e)}", 400)
        
        if workflow is None:
            raise LangChainException("The LangChain Expression Language Agent Workflow object provided is invalid.", 400)

        return workflow



from enum import Enum

class LanguageModelProvider(str, Enum):
    """Enumerator of the Language Model providers."""
    AZUREAI = "azureai"
    MICROSOFT = "microsoft"
    OPENAI = "openai"
    BEDROCK = "bedrock"
    VERTEXAI = "vertexai"
    DATABRICKS = "databricks"



from typing import Optional

from langchain_community.tools.sql_database.tool import BaseSQLDatabaseTool
from langchain_core.callbacks import CallbackManagerForToolRun
from langchain_core.pydantic_v1 import Field
from langchain_core.tools import BaseTool

class SecureSQLDatabaseQueryTool(BaseSQLDatabaseTool, BaseTool):
    """Tool for querying a SQL database."""

    name: str = "secure_sql_db_query"
    description: str = """
        Input to this tool is a detailed and correct SQL query, output is a result from the database.
        If the query is not correct, an error message will be returned.
        If an error is returned, rewrite the query, check the query, and try again.
        """

    username: str = Field(exclude=True)
    use_row_level_security: bool = Field(exclude=True)

    def _run(
        self,
        query: str,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> str:
        """
        Execute the query, conditionally wrapping it in MSSQL row-level security statements.
        Return the results or an error message.
        """
        if (self.use_row_level_security) and (self.db.dialect=='mssql') and (self.username!=''):
            if (query.startswith(f"EXECUTE AS USER = '{self.username}';")) \
                and (query.endswith('REVERT;')):
                query = query
            else:
                query = (
                    f"EXECUTE AS USER = '{self.username}'; "
                    f"{query}; "
                    "REVERT;"
                )
        else:
            query = query

        return self.db.run_no_throw(query)



from typing import Optional
from foundationallm.models.resource_providers import ResourceBase

class Attachment(ResourceBase):
    """
    Encapsulates the attachment model from resource provider.
    """
    content_type: Optional[str] = None
    path: str
